#ifndef _DAMLDB_TIME_VALUE_HH
#define _DAMLDB_TIME_VALUE_HH

#if defined (sun)
#  pragma ident "@(#)$Id: Time_Value.hh,v 1.2 2002/04/09 02:04:46 neves Exp $"
#endif

#include "damldb/Export.hh"
#include "damldb/Portability.hh"

#ifdef CCXX_NAMESPACES
namespace damldb {
#endif

  /**
     Time_Value encapsulates the timeval structure for portability.

     \begin{description}
     \item[CREDIT] Doug Schmidt <schmidt@cs.wustl.edu> ACE Library
     \item[NOTE] This interface was derived from the ACE_Time_Value Class.
     It has been locally modified to extract it from ACE.
     \end{description}
  */

  class __EXPORT Time_Value
  {
  public:
    // Constant "0".
    static const Time_Value zero;

    /** @name Initialization methods. */
    //@{
    /// Default Constructor.
    Time_Value (void);

    /// Constructor.
    Time_Value (long sec, long usec = 0);
    //@}

    /** @name Methods for converting to/from various time formats. */
    //@{
    /// Construct the <Time_Value> from a <timeval>.
    Time_Value (const struct timeval &t);

    /// Copy constructor.
    Time_Value (const Time_Value &tv);

    /// Construct a <Time_Value> from two <long>s.
    void set (long sec, long usec);

    /**
       Construct a <Time_Value> from a <double>, which is assumed to be
       in second format, with any remainder treated as microseconds.
    */
    void set (double d);

    /// Construct a <Time_Value> from a <timeval>.
    void set (const timeval &t);

    /// Converts from <Time_Value> format into milli-seconds format.
    long msec (void) const;

    /// Converts from milli-seconds format into <Time_Value> format.
    void msec (long);

    /// Returns the value of the object as a <timeval>.
    operator timeval () const;

    /// Returns a pointer to the object as a <timeval>.
    operator const timeval *() const;
    //@}

    /** @name accessor/mutator methods. */
    //@{
    /// Get seconds.
    long sec (void) const;

    /// Set seconds.
    void sec (long sec);

    /// Get microseconds.
    long usec (void) const;

    /// Set microseconds.
    void usec (long usec);
    //@}

    /** @name arithmetic methods */
    //@{
    /// Add <tv> to this.
    void operator += (const Time_Value &tv);

    /// Subtract <tv> to this.
    void operator -= (const Time_Value &tv);
    //@}

    /// Adds two Time_Value objects together, returns the sum.
    friend __EXPORT Time_Value operator + (const Time_Value &tv1, const Time_Value &tv2);

    /// Subtracts two Time_Value objects, returns the difference.
    friend __EXPORT Time_Value operator - (const Time_Value &tv1, const Time_Value &tv2);

    /// True if tv1 < tv2.
    friend __EXPORT int operator < (const Time_Value &tv1, const Time_Value &tv2);

    /// True if tv1 > tv2.
    friend __EXPORT int operator > (const Time_Value &tv1, const Time_Value &tv2);

    /// True if tv1 <= tv2.
    friend __EXPORT int operator <= (const Time_Value &tv1, const Time_Value &tv2);

    /// True if tv1 >= tv2.
    friend __EXPORT int operator >= (const Time_Value &tv1, const Time_Value &tv2);

    /// True if tv1 == tv2.
    friend __EXPORT int operator == (const Time_Value &tv1, const Time_Value &tv2);

    /// True if tv1 != tv2.
    friend __EXPORT int operator != (const Time_Value &tv1, const Time_Value &tv2);

    /** @name debug methods */
    //@{
    /// Dump the state of an object.
    void dump (void) const;
    //@}

  private:
    /// Put the timevalue into a canonical form.
    void normalize (void);

    /// Store the values as a <timeval>.
    timeval tv_;
  };

  /** @name Time Constants */
  //@{
  /// Number of milliseconds in one second
  const long DAMLDB_ONE_SECOND_IN_MSECS = 1000L;

  /// Number of microseconds in one second
  const long DAMLDB_ONE_SECOND_IN_USECS = 1000000L;

  /// Number of nanoseconds in one second
  const long DAMLDB_ONE_SECOND_IN_NSECS = 1000000000L;
  //@}

#ifdef CCXX_NAMESPACES
} //namespace damldb
#endif

#ifndef DAMLDB_OUTLINE
#include "Time_Value.icc"
#endif

#endif /* !_DAMLDB_TIME_VALUE_HH */

// Emacs
// Local Variables:
// mode: c++
// c-basic-offset: 2
// End:
