
#ifndef lint
static const char revid[] = 
	"@(#)$Id: DamlDBImpl.cpp,v 1.3 2002/04/10 21:23:35 neves Exp $";
#endif

#include "damldb/DamlDBImpl.hh"
#include "damldb/OS.hh"

#include <algorithm>
#include <iostream>
#include <stdexcept>
#include <string>

#ifdef CCXX_NAMESPACES
namespace damldb {
#endif

  const string kINT2STRING_DB = string("i2s.db");
  const string kSTRING2INT_DB = string("s2i.db");

  ////////////////////////////////////////////////////////////

  class dump_statement_record {
  public:
    dump_statement_record(ostream& os) : os_(os), i_(0) {}
    int operator()(const Statement x) const
    {
      os_ << endl;
      os_ << "statement[" << i_ << "].flags           = " << x.flags << endl;
      os_ << "statement[" << i_ << "].source          = " << x.source << endl;
      os_ << "statement[" << i_ << "].sourceNext      = " << x.sourceNext << endl;
      os_ << "statement[" << i_ << "].subject         = " << x.subject << endl;
      os_ << "statement[" << i_ << "].subjectNext     = " << x.subjectNext << endl;
      os_ << "statement[" << i_ << "].predicate       = " << x.predicate << endl;
      os_ << "statement[" << i_ << "].predicateNext   = " << x.predicateNext << endl;
      os_ << "statement[" << i_ << "].object          = " << x.object << endl;
      os_ << "statement[" << i_ << "].objectNext      = " << x.objectNext << endl;

      i_++;
      return 0;
    }
  private:
    mutable ostream& os_;
    mutable int i_;
  };

  ////////////////////////////////////////////////////////////

  class dump_resource_record {
  public:
    dump_resource_record(ostream& os) : os_(os), i_(0) {}
    int operator()(const Resource x) const
    {
      os_ << endl;
      os_ << "resource" << i_ << "].flags           = " << x.flags << endl;
      os_ << "resource" << i_ << "].sourceFirst      = " << x.sourceFirst << endl;
      os_ << "resource" << i_ << "].subjectFirst     = " << x.subjectFirst << endl;
      os_ << "resource" << i_ << "].predicateFirst   = " << x.predicateFirst << endl;
      os_ << "resource" << i_ << "].objectFirst      = " << x.objectFirst << endl;

      i_++;
      return 0;
    }
  private:
    mutable ostream& os_;
    mutable int i_;
  };


  ////////////////////////////////////////////////////////////

  DamlDBImpl::DamlDBImpl(const string& directory,
			 int open_mode,
			 stmt_mapped_allocator& stmt_alloc,
			 res_mapped_allocator& res_alloc)
    : _statementDB(0, stmt_alloc),
      _resourceDB(0, res_alloc),
      _i2s(directory + OS::filesep() + kINT2STRING_DB, open_mode),
      _s2i(directory + OS::filesep() + kSTRING2INT_DB, open_mode)
  {}

  ////////////////////////////////////////////////////////////

  DamlDBImpl::~DamlDBImpl()
  {
    sync();
  }

  ////////////////////////////////////////////////////////////

  void
  DamlDBImpl::add_resource()
  {
    Resource resource;

    resource.flags	    = RESOURCE_FLAG_VALID;
    resource.sourceFirst    = NULL_STATEMENT_ID;
    resource.subjectFirst   = NULL_STATEMENT_ID;
    resource.predicateFirst = NULL_STATEMENT_ID;
    resource.objectFirst    = NULL_STATEMENT_ID;

    add_resource(resource);
  }

  ////////////////////////////////////////////////////////////

  void
  DamlDBImpl::add_resource(const Resource& resource)
  {
    _resourceDB.push_back(resource);
  }

  ////////////////////////////////////////////////////////////

  DamlDBImpl::size_type
  DamlDBImpl::get_resource_count() const
  {
    return _resourceDB.size();
  }

  ////////////////////////////////////////////////////////////

  const Resource&
  DamlDBImpl::get_resource(const ResourceID& id) const
  {
    if (id == NULL_RESOURCE_ID || id >= _resourceDB.size())
      {
	THROW(runtime_error("DamlDBImpl::get_resource called with invalid resource ID"));
      }

    return _resourceDB[id];
  }

  ////////////////////////////////////////////////////////////

  const Resource&
  DamlDBImpl::get_resource(const char* uri)
  {
    return _resourceDB[uri2resource(uri)];
  }

  ////////////////////////////////////////////////////////////

  const Statement&
  DamlDBImpl::add_statement(const char* sourceURI,
			    const char* subjectURI,
			    const char* predicateURI,
			    const char* objectURI)
  {
    return add_statement(uri2resource(sourceURI),
			 uri2resource(subjectURI),
			 uri2resource(predicateURI),
			 uri2resource(objectURI));
  }

  ////////////////////////////////////////////////////////////

  const Statement&
  DamlDBImpl::add_statement(const ResourceID& sourceID,
			    const ResourceID& subjectID,
			    const ResourceID& predicateID,
			    const ResourceID& objectID)
  {
    size_type next_statementID = _statementDB.size();

    Statement statement;

    statement.flags = STATEMENT_FLAG_VALID;

    // Set up Statement source, subject, predicate, and object
    statement.source    = sourceID;
    statement.subject   = subjectID;
    statement.predicate = predicateID;
    statement.object    = objectID;

    // Link statement into source linked list
    Resource& sourceResource		= _resourceDB[sourceID];
    statement.sourceNext		= sourceResource.sourceFirst;
    sourceResource.sourceFirst		= next_statementID;

    // Link statement into subject linked list
    Resource& subjectResource		= _resourceDB[subjectID];
    statement.subjectNext		= subjectResource.subjectFirst;
    subjectResource.subjectFirst	= next_statementID; 

    // Link statement into predicate linked list
    Resource& predicateResource		= _resourceDB[predicateID];
    statement.predicateNext		= predicateResource.predicateFirst;
    predicateResource.predicateFirst	= next_statementID;

    // Link statement into object linked list
    Resource& objectResource		= _resourceDB[objectID];
    statement.objectNext		= objectResource.objectFirst;
    objectResource.objectFirst		= next_statementID;

    return add_statement(statement);
  }

  ////////////////////////////////////////////////////////////

  const Statement&
  DamlDBImpl::add_statement(const Statement& statement)
  {
    _statementDB.push_back(statement); 
    return *_statementDB.end();
  }

  ////////////////////////////////////////////////////////////

  DamlDBImpl::size_type
  DamlDBImpl::get_statement_count() const
  {
    return _statementDB.size();
  }

  ////////////////////////////////////////////////////////////

  const Statement&
  DamlDBImpl::get_statement(const StatementID& id) const
  {
    if (id == NULL_STATEMENT_ID || id >= _statementDB.size())
      {
	THROW(runtime_error("DamlDBImpl::get_statement called with invalid statementID"));
      }

    return _statementDB[id];
  }

  ////////////////////////////////////////////////////////////

  const string&
  DamlDBImpl::resource2uri(const ResourceID& resource, string& result)
  {
    return _i2s.find(resource, result);
  }

  ////////////////////////////////////////////////////////////

  ResourceID
  DamlDBImpl::uri2resource(const char* uri)
  {
    ResourceID retVal;
    string sUri(uri);

    try
      {
	StringIntMap::value_type tmp_id;
	retVal = _s2i.find(sUri, tmp_id);
      }
    catch( const exception& x)
      {
	const ResourceID id = _resourceDB.size();

	add_resource();
	update_indices(sUri, id);

	retVal = id;
      }

    return retVal;
  }

  ////////////////////////////////////////////////////////////

  void
  DamlDBImpl::sync()
  {
    _i2s.sync();
    _s2i.sync();
  }

  ////////////////////////////////////////////////////////////

  void
  DamlDBImpl::dump_statements(ostream& os) const
  {
    dump_statement_record functor(os);

    for_each<StatementDB::const_iterator,dump_statement_record>
      (_statementDB.begin(), _statementDB.end(), functor);
  }

  ////////////////////////////////////////////////////////////

  void
  DamlDBImpl::dump_resources(ostream& os) const
  {
    dump_resource_record functor(os);

    for_each<ResourceDB::const_iterator, dump_resource_record>
      (_resourceDB.begin(), _resourceDB.end(), functor);
  }

  ////////////////////////////////////////////////////////////

  void
  DamlDBImpl::update_indices(const string& s, const ResourceID& id)
  {
    _s2i.insert(s, id);
    _i2s.insert(id, s);
  }

#ifdef CCXX_NAMESPACES
} // namespace damldb
#endif

// Emacs
// Local Variables:
// mode: c++
// c-basic-offset: 2
// End:
